#include "ros/ros.h"
#include "std_msgs/String.h"

#include "geometry_msgs/Twist.h"

#include "turtlesim/Pose.h"

#define PI 3.14159265359

ros::Publisher velocity_pub;
turtlesim::Pose current_pose;

void pose_cb(const turtlesim::Pose::ConstPtr& msg){
	current_pose = *msg;
	//ROS_INFO_STREAM(current_pose);
}

bool closeEnough(float angle1, float angle2, float eps){
	float d1 = fabs(angle1 - angle2);
	float d2 = fabs(angle1 - angle2);
	
	ROS_INFO("%f, %f",d1,d2);
	
	if (d1 < eps || d2 > (2*PI - eps)){
		return true;
	}
	else {
		return false;
	}
}

void turn(float angular_vel, float desired_orientation){
	geometry_msgs::Twist v;
	
	v.linear.x = 0.0;
	v.linear.y = 0.0;
	v.linear.z = 0.0;
	v.angular.x = 0.0;
	v.angular.y = 0.0;
	v.angular.z = angular_vel;
	
	float framerate = 40.0;
	ros::Rate loop_rate(framerate);
	
	bool reached = false;
	
	while (!reached){
		//publish the velocity command
		velocity_pub.publish(v);
		
		//listen for incomming messages
		ros::spinOnce();
		
		if (closeEnough(desired_orientation,  
						current_pose.theta,PI/10.0)){
			reached = true;				
		}
		
		loop_rate.sleep();
	}
}



void moveForward(float v_x, float duration){
	geometry_msgs::Twist v;
	
	v.linear.x = v_x;
	v.linear.y = 0.0;
	v.linear.z = 0.0;
	v.angular.x = 0.0;
	v.angular.y = 0.0;
	v.angular.z = 0.0;

	float framerate = 40.0;
	ros::Rate loop_rate(framerate);
	
	
	for (float t = 0; t < duration; t += 1.0/framerate){
		velocity_pub.publish(v);
		loop_rate.sleep();
		ROS_INFO("Moving forward....");
	}
}

int main(int argc, char **argv){
	
	
	ros::init(argc, argv, "turtle_sim_controller");
	ros::NodeHandle n;
	
	velocity_pub = n.advertise<geometry_msgs::Twist>("/turtle1/cmd_vel", 1);

	ros::Subscriber sub_pose = n.subscribe("/turtle1/pose", 1, pose_cb );
   


	moveForward(2.0,1.0);
	turn(0.25,-PI); 
	
	
	
	return 0;
}
